using System;
using System.Collections.Generic;
using System.IO;
using System.Security.Cryptography;
using System.Text;

namespace Vevo.Shared.Utilities
{
    public static class SymmetricEncryption
    {
        #region private

        private static string _saltValue = "s@Vevo";               // can be any string
        private static string _hashAlgorithm = "SHA1";             // can be "MD5"
        private static int _passwordIterations = 2;                // can be any number
        private static string _initVector = "@1B2c3D4e5F6g7H8";    // must be 16 bytes
        private static int _keySize = 256;                         // can be 192 or 128
        private static int _lenghtIV = 24;

        #endregion


        #region Constructors

        #endregion


        #region Public Methods

        public static string Encrypt( string plainText, string secretKey )
        {
            if (String.IsNullOrEmpty( plainText ))
                return String.Empty;

            byte[] initVectorBytes = Encoding.ASCII.GetBytes( _initVector );
            byte[] saltValueBytes = Encoding.ASCII.GetBytes( _saltValue );

            byte[] plainTextBytes = Encoding.UTF8.GetBytes( plainText );
            PasswordDeriveBytes password = new PasswordDeriveBytes( secretKey, saltValueBytes, _hashAlgorithm, _passwordIterations );

            byte[] keyBytes = password.GetBytes( _keySize / 8 );

            RijndaelManaged symmetricKey = new RijndaelManaged();
            symmetricKey.Mode = CipherMode.CBC;

            ICryptoTransform encryptor = symmetricKey.CreateEncryptor( keyBytes, initVectorBytes );

            MemoryStream memoryStream = new MemoryStream();

            CryptoStream cryptoStream = new CryptoStream( memoryStream, encryptor, CryptoStreamMode.Write );

            cryptoStream.Write( plainTextBytes, 0, plainTextBytes.Length );

            cryptoStream.FlushFinalBlock();

            byte[] cipherTextBytes = memoryStream.ToArray();

            memoryStream.Close();
            cryptoStream.Close();

            string cipherText = Convert.ToBase64String( cipherTextBytes );

            return Convert.ToBase64String( symmetricKey.IV ).Substring( 0, _lenghtIV - 1 ) + cipherText;
        }

        public static string Decrypt( string cipherText, string secretKey )
        {
            if (String.IsNullOrEmpty( cipherText ))
                return String.Empty;
            try
            {
                cipherText = cipherText.Substring( _lenghtIV - 1 );
                byte[] initVectorBytes = Encoding.ASCII.GetBytes( _initVector );
                byte[] saltValueBytes = Encoding.ASCII.GetBytes( _saltValue );

                byte[] cipherTextBytes = Convert.FromBase64String( cipherText );

                PasswordDeriveBytes password = new PasswordDeriveBytes( secretKey, saltValueBytes, _hashAlgorithm, _passwordIterations );

                byte[] keyBytes = password.GetBytes( _keySize / 8 );

                RijndaelManaged symmetricKey = new RijndaelManaged();

                symmetricKey.Mode = CipherMode.CBC;

                ICryptoTransform decryptor = symmetricKey.CreateDecryptor( keyBytes, initVectorBytes );

                MemoryStream memoryStream = new MemoryStream( cipherTextBytes );

                CryptoStream cryptoStream = new CryptoStream( memoryStream, decryptor, CryptoStreamMode.Read );

                byte[] plainTextBytes = new byte[cipherTextBytes.Length];

                int decryptedByteCount = cryptoStream.Read( plainTextBytes, 0, plainTextBytes.Length );

                memoryStream.Close();
                cryptoStream.Close();

                string plainText = Encoding.UTF8.GetString( plainTextBytes, 0, decryptedByteCount );

                return plainText;
            }
            catch
            {
                return String.Empty;
            }
        }

        public static string Decrypt( object cipherText, string secretKey )
        {
            return Decrypt( ConvertUtilities.ToString( cipherText ), secretKey );
        }

        #endregion
    }
}
