using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;

namespace Vevo.DataAccessLib.Cart
{
    public class AdminMenuAdvancedDetails
    {
        private string _menuKey;
        private string _parentMenuKey;
        private string _pageName;
        private int _sortOrder;
        private bool _isEnabled;
        private string _menuName;
        private string _menuDescription;

        public string MenuKey { get { return _menuKey; } set { _menuKey = value; } }
        public string ParentMenuKey { get { return _parentMenuKey; } set { _parentMenuKey = value; } }
        public string PageName { get { return _pageName; } set { _pageName = value; } }
        public int SortOrder { get { return _sortOrder; } set { _sortOrder = value; } }
        public bool IsEnabled { get { return _isEnabled; } set { _isEnabled = value; } }
        public string MenuName { get { return _menuName; } set { _menuName = value; } }
        public string MenuDescription { get { return _menuDescription; } set { _menuDescription = value; } }
    }

    public static class AdminMenuAdvancedAccess
    {
        private const string AllMenuFields = "AdminMenuAdvanced.MenuKey, ParentMenuKey, PageName, SortOrder, " +
            "IsEnabled, MenuName, MenuDescription, MenuClass  ";

        private static AdminMenuAdvancedDetails ConvertDataRowToDetails( DataRow row )
        {
            AdminMenuAdvancedDetails details = new AdminMenuAdvancedDetails();
            details.MenuKey = row["MenuKey"].ToString();
            details.ParentMenuKey = row["ParentMenuKey"].ToString();
            details.PageName = row["PageName"].ToString();
            details.SortOrder = ConvertUtilities.ToInt32( row["SortOrder"] );
            details.IsEnabled = ConvertUtilities.ToBoolean( row["IsEnabled"] );
            details.MenuName = row["MenuName"].ToString();
            details.MenuDescription = row["MenuDescription"].ToString();
            return details;
        }

        public static DataTable GetAll()
        {
            return GetAll( "1", "SortOrder", FlagFilter.ShowAll );
        }

        public static DataTable GetAll( string cultureID, string sortBy, FlagFilter isEnabled )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllMenuFields + " FROM " +
                "AdminMenuAdvanced LEFT JOIN AdminMenuAdvancedLocale ON " +
                "AdminMenuAdvanced.MenuKey = AdminMenuAdvancedLocale.MenuKey " +
                "WHERE CultureID = @CultureID " +
                FlagFilterHelper.CreateFlagFilterWithAnd( "IsEnabled", isEnabled ) +
                CreateSortByText( sortBy ),
                DataAccess.CreateParameterString( cultureID ) );
            return table;
        }

        public static DataTable GetAllByAdminID(
            string cultureID,
            string adminID,
            string sortBy,
            FlagFilter isEnabled )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllMenuFields + ", ViewMode, ModifyMode FROM " +
                "( AdminMenuAdvanced LEFT JOIN AdminMenuAdvancedLocale ON " +
                "AdminMenuAdvanced.MenuKey = AdminMenuAdvancedLocale.MenuKey ) " +
                "LEFT JOIN AdminMenuPermission ON " +
                "AdminMenuAdvanced.PageName = AdminMenuPermission.MenuPageName " +
                "WHERE CultureID = @CultureID AND AdminID = @AdminID " +
                FlagFilterHelper.CreateFlagFilterWithAnd( "IsEnabled", isEnabled ) +
                CreateSortByText( sortBy ),
                DataAccess.CreateParameterString( cultureID ),
                DataAccess.CreateParameterString( adminID ) );
            return table;
        }

        public static AdminMenuAdvancedDetails GetDetails( string cultureID, string menuKey )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllMenuFields + " FROM " +
                "AdminMenuAdvanced LEFT JOIN AdminMenuAdvancedLocale ON " +
                "AdminMenuAdvanced.MenuKey = AdminMenuAdvancedLocale.MenuKey " +
                "WHERE CultureID = @CultureID AND AdminMenuAdvanced.MenuKey = @MenuKey",
                DataAccess.CreateParameterString( cultureID ),
                DataAccess.CreateParameterString( menuKey ) );
            if (table.Rows.Count > 0)
                return ConvertDataRowToDetails( table.Rows[0] );
            else
                return new AdminMenuAdvancedDetails();
        }

        private static string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else if (sortBy.Trim().ToLower().StartsWith( "menukey" ))
                return "ORDER BY AdminMenuAdvanced." + sortBy.Trim() + " ";
            else
                return "ORDER BY " + sortBy + " ";
        }

        public static string Create(
             string cultureID,
             string menuKey,
             string parentMenuKey,
             string pageName,
             string menuName,
             string menuDescription,
             int sortOrder,
             bool isEnabled
             )
        {
            string menuID = DataAccess.ExecuteScalar(
                "INSERT INTO AdminMenuAdvanced ( MenuKey, ParentMenuKey, PageName, SortOrder, IsEnabled ) " +
                "VALUES ( @MenuKey, @ParentMenuKey, @PageName, @SortOrder, @IsEnabled ); " +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( menuKey ),
                DataAccess.CreateParameterString( parentMenuKey ),
                DataAccess.CreateParameterString( pageName ),
                DataAccess.CreateParameterInt32( sortOrder ),
                DataAccess.CreateParameterBool( isEnabled )
                );

            DataAccess.ExecuteNonQuery(
                "INSERT INTO AdminMenuAdvancedLocale ( CultureID, MenuKey, MenuName, MenuDescription ) " +
                "VALUES (" +
                " @CultureID, @MenuPageName, @MenuName, @MenuDescription );",
            DataAccess.CreateParameterString( cultureID ),
            DataAccess.CreateParameterString( menuKey ),
            DataAccess.CreateParameterString( menuName ),
            DataAccess.CreateParameterString( menuDescription ) );

            return menuID;
        }
    }
}
