using System;
using System.Collections.Generic;
using System.Data;
using System.Text;
using Vevo.Domain;
using Vevo.Domain.Orders;
using Vevo.Shared.Utilities;
using Vevo.Shared.DataAccess;

namespace Vevo.Data.Implementation
{
    internal class OrderItemRecurringAccess
    {
        #region Private

        private OrderItemRecurring CreateOneFromRow( DataRow row )
        {
            OrderItemRecurring orderItemRecurring = new OrderItemRecurring();
            orderItemRecurring.OrderItemID = row["OrderItemID"].ToString();
            orderItemRecurring.RecurringStatus = row["RecurringStatus"].ToString();
            orderItemRecurring.RecurringPeriod = row["RecurringPeriod"].ToString();
            orderItemRecurring.RecurringReceiptID = row["RecurringReceiptID"].ToString();
            orderItemRecurring.UpdateTime = ConvertUtilities.ToDateTime( row["UpdateTime"] );

            return orderItemRecurring;
        }

        #endregion


        #region Public Methods

        public OrderItemRecurring Create( OrderItemRecurring orderItemRecurring )
        {
            DataAccess.ExecuteNonQuery(
                "INSERT INTO OrderItemRecurring (OrderItemID, RecurringStatus, RecurringPeriod, RecurringReceiptID, UpdateTime ) " +
                "VALUES (@OrderItemID, @RecurringStatus, @RecurringPeriod, @RecurringReceiptID, @UpdateTime);",
                DataAccess.CreateParameterString( orderItemRecurring.OrderItemID ),
                DataAccess.CreateParameterString( orderItemRecurring.RecurringStatus ),
                DataAccess.CreateParameterString( orderItemRecurring.RecurringPeriod ),
                DataAccess.CreateParameterString( orderItemRecurring.RecurringReceiptID ),
                DataAccess.CreateParameterDateTime( orderItemRecurring.UpdateTime )
                );
            return orderItemRecurring;
        }

        public void Update( OrderItemRecurring orderItemRecurring )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE OrderItemRecurring " +
                "SET RecurringStatus = @RecurringStatus, " +
                "RecurringPeriod = @RecurringPeriod, " +
                "UpdateTime = @UpdateTime " +
                "WHERE  OrderItemID = @OrderItemID;",
                DataAccess.CreateParameterString( orderItemRecurring.RecurringStatus ),
                DataAccess.CreateParameterString( orderItemRecurring.RecurringPeriod ),
                DataAccess.CreateParameterDateTime( orderItemRecurring.UpdateTime ),
                DataAccess.CreateParameterString( orderItemRecurring.OrderItemID ) );
        }

        public void Delete( string orderItemID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM OrderItemRecurring " +
                "WHeRE OrderItemID = @OrderItemID; ",
                DataAccess.CreateParameterString( orderItemID ) );
        }

        public OrderItemRecurring GetOne( string orderItemID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT OrderItemID, RecurringStatus, RecurringPeriod, RecurringReceiptID, UpdateTime " +
                "FROM OrderItemRecurring " +
                "WHERE OrderItemID = @OrderItemID; ",
                 DataAccess.CreateParameterString( orderItemID )
                 );
            if (table.Rows.Count > 0)
                return CreateOneFromRow( table.Rows[0] );
            else
                return null;
        }

        #endregion
    }
}
