using System;
using System.Collections.Generic;
using Vevo.Domain.Base;
using Vevo.Domain.Users;

namespace Vevo.Domain.Contents
{
    public class Content : ILocaleLoader, ICloneable
    {
        #region Private

        private string _contentID = "0";
        private string _contentName = String.Empty;
        private bool _isShowInSiteMap = true;
        private bool _isEnabled = true;
        private string _other1 = String.Empty;
        private string _other2 = String.Empty;
        private string _other3 = String.Empty;
        private string _subscriptionLevelID = "0";

        private Culture _defaultCulture;
        private LocaleCollection<ContentLocale> _locales;

        #endregion

        #region Public Constructors

        public Content( Culture defaultCulture )
        {
            _defaultCulture = defaultCulture;
            _locales = new LocaleCollection<ContentLocale>( this );
        }

        #endregion

        #region Public Properties

        public string ContentID { get { return _contentID; } set { _contentID = value; } }
        public string ContentName { get { return _contentName; } set { _contentName = value; } }
        public bool IsEnabled { get { return _isEnabled; } set { _isEnabled = value; } }
        public bool IsShowInSiteMap { get { return _isShowInSiteMap; } set { _isShowInSiteMap = value; } }
        public string Other1 { get { return _other1; } set { _other1 = value; } }
        public string Other2 { get { return _other2; } set { _other2 = value; } }
        public string Other3 { get { return _other3; } set { _other3 = value; } }
        public string SubscriptionLevelID { get { return _subscriptionLevelID; } set { _subscriptionLevelID = value; } }

        public Culture DefaultCulture { get { return _defaultCulture; } set { _defaultCulture = value; } }
        public LocaleCollection<ContentLocale> Locales { get { return _locales; } }

        public static Content Null
        {
            get { return NullContent.Instance; }
        }

        public virtual bool IsNull
        {
            get { return false; }
        }


        //------------------------------------------------------------
        // Locale items
        public string Title
        {
            get { return Locales[DefaultCulture].Title; }
            set { Locales.GetNoLoad( DefaultCulture ).Title = value; }
        }

        public string Body
        {
            get { return Locales[DefaultCulture].Body; }
            set { Locales.GetNoLoad( DefaultCulture ).Body = value; }
        }

        public string MetaTitle
        {
            get { return Locales[DefaultCulture].MetaTitle; }
            set { Locales.GetNoLoad( DefaultCulture ).MetaTitle = value; }
        }

        public string MetaKeyword
        {
            get { return Locales[DefaultCulture].MetaKeyword; }
            set { Locales.GetNoLoad( DefaultCulture ).MetaKeyword = value; }
        }

        public string MetaDescription
        {
            get { return Locales[DefaultCulture].MetaDescription; }
            set { Locales.GetNoLoad( DefaultCulture ).MetaDescription = value; }
        }
        public string UrlName
        {
            get { return Locales[ DefaultCulture ].UrlName; }
            set { Locales.GetNoLoad( DefaultCulture ).UrlName = value; }
        }
        // End Locale items
        //------------------------------------------------------------
        #endregion

        #region ILocaleLoader Members

        public IList<ILocale> GetLocales()
        {
            List<ILocale> result = new List<ILocale>();

            foreach (ContentLocale locale in DataAccessContext.ContentRepository.GetLocales( ContentID ))
                result.Add( locale );

            return result;
        }

        #endregion


        #region IClonable Members

        // Copy the whole aggregate (including locale)
        public object Clone()
        {
            Content newObject = (Content) MemberwiseClone();
            newObject._locales = (LocaleCollection<ContentLocale>) Locales.Clone();
            return newObject;
        }

        #endregion


        #region Public Methods


        public string GetMetaDescription( Culture culture )
        {
            if (!String.IsNullOrEmpty( Locales[culture].MetaDescription ))
                return Locales[culture].MetaDescription;
            else
                return Locales[culture].Title;
        }

        #endregion
    }
}
