using System;
using System.Collections.Generic;
using System.Text;

using Vevo.Domain.Users;

namespace Vevo.Domain.Orders
{
    public class OrderValidator
    {
        public bool ValidateSubtotal(
            Currency currency,
            WholesaleStatus wholesaleStatus,
            Cart cart,
            out string errorHeader,
            out string errorBody )
        {
            errorHeader = String.Empty;
            errorBody = String.Empty;

            if (DataAccessContext.Configurations.GetBoolValue( "OrderMaximumEnabled" ) ||
                DataAccessContext.Configurations.GetBoolValue( "OrderMinimumEnabled" ))
            {
                decimal subtotal = cart.GetSubtotal( wholesaleStatus );
                decimal orderMaximumAmount = DataAccessContext.Configurations.GetDecimalValue( "OrderMaximumAmount" );
                decimal orderMinimumAmount = DataAccessContext.Configurations.GetDecimalValue( "OrderMinimumAmount" );

                if (DataAccessContext.Configurations.GetBoolValue( "OrderMaximumEnabled" )
                    && subtotal > orderMaximumAmount)
                {
                    errorHeader = "[$ErrorOrderAmountHeader]";
                    errorBody = String.Format( "[$ErrorOrderAmountLimitMax] {0}",
                        currency.FormatPrice( orderMaximumAmount ) );
                    return false;
                }
                else if (DataAccessContext.Configurations.GetBoolValue( "OrderMinimumEnabled" )
                    && subtotal < orderMinimumAmount)
                {
                    errorHeader = "[$ErrorOrderAmountHeader]";
                    errorBody = String.Format( "[$ErrorOrderAmountLimitMin] {0}",
                        currency.FormatPrice( orderMinimumAmount ) );
                    return false;
                }
            }

            return true;
        }
    }
}
