﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using Vevo.WebServices.Base;
using Vevo.WebServices.Security;
using Vevo.WebServices.SearchFilters;
using Vevo.WebAppLib;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain;
using System.Data;
using Vevo.Domain.Marketing;
using Vevo.Base.Domain;
using Vevo.Deluxe.Domain.Marketing;
using Vevo.Deluxe.Domain;

namespace Vevo.WebServices.Affiliates
{
    public class AffiliateService : IAffiliateService
    {
        private AffiliateConstraints ConvertAffiliateToConstraints( Affiliate affiliate )
        {
            AffiliateConstraints constraints = new AffiliateConstraints();
            constraints.AffiliateCode = affiliate.AffiliateCode;
            constraints.CommissionRate = affiliate.CommissionRate;
            constraints.Email = affiliate.Email;
            constraints.IsEnabled = affiliate.IsEnabled;
            constraints.RegisterDate = affiliate.RegisterDate;
            constraints.UserName = affiliate.UserName;
            constraints.Website = affiliate.Website;

            constraints.ContactAddress = new ContactAddressConstraints();
            constraints.ContactAddress.Address1 = affiliate.ContactAddress.Address1;
            constraints.ContactAddress.Address2 = affiliate.ContactAddress.Address2;
            constraints.ContactAddress.City = affiliate.ContactAddress.City;
            constraints.ContactAddress.Company = affiliate.ContactAddress.Company;
            constraints.ContactAddress.Country = affiliate.ContactAddress.Country;
            constraints.ContactAddress.Fax = affiliate.ContactAddress.Fax;
            constraints.ContactAddress.FirstName = affiliate.ContactAddress.FirstName;
            constraints.ContactAddress.LastName = affiliate.ContactAddress.LastName;
            constraints.ContactAddress.Phone = affiliate.ContactAddress.Phone;
            constraints.ContactAddress.State = affiliate.ContactAddress.State;
            constraints.ContactAddress.Zip = affiliate.ContactAddress.Zip;

            return constraints;
        }

        public AffiliateListResponseMessage ListAffiliates( AffiliateListRequestMessage ListMessage )
        {
            AffiliateListResponseMessage responseMessage = new AffiliateListResponseMessage();
            if ( WcfSessionPool.GetSession( ListMessage.Ticket ) == null )
            {
                responseMessage.ErrorMessage = "Error: Authentication is failed.";
                return responseMessage;
            }
            try
            {
                SearchFilter searchFilter = SearchFilterUtilities.ConvertConstraintToSearchFilter( ListMessage.SearchFilter );
                int affiliateCount = 0;
                IList<Affiliate> affiliate_result = DataAccessContextDeluxe.AffiliateRepository.SearchAffiliate( "AffiliateCode", searchFilter, ListMessage.SearchFilter.StartIndex, ListMessage.SearchFilter.EndIndex, out affiliateCount );
                responseMessage.Affiliates = new List<AffiliateConstraints>();
                foreach ( Affiliate affiliate in affiliate_result )
                {
                    responseMessage.Affiliates.Add( ConvertAffiliateToConstraints( affiliate ) );
                }
            }
            catch ( Exception e )
            {
                WebUtilities.LogError( e );
                responseMessage.ErrorMessage = "Error: Cannot get affiliates from repository";
            }

            return responseMessage;
        }
    }
}
